"""Configuration managed by the system. Do not make changes you aren't
afraid of being overwritten and removed automatically."""

SYS_TEST_MESSAGE = "Hi! Jubeo taskset author here, please don't override these options " \
                   "unless you know what you are doing."
