"""User settings for a project."""

# load the system configuration. You can override them in this module,
# but beware it might break stuff
from .sysconfig import *

TEST_MESSAGE = "You can configure modules from this file"
